from PyObjCTools.TestSupport import TestCase
import ModelIO
from objc import simd


class TestMDLCamera(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(ModelIO.MDLCameraProjection)

    def testConstants(self):
        self.assertEqual(ModelIO.MDLCameraProjectionPerspective, 0)
        self.assertEqual(ModelIO.MDLCameraProjectionOrthographic, 1)

    def testMethods(self):
        self.assertResultHasType(
            ModelIO.MDLCamera.projectionMatrix, simd.matrix_float4x4.__typestr__
        )

        self.assertArgHasType(
            ModelIO.MDLCamera.frameBoundingBox_setNearAndFar_,
            0,
            ModelIO.MDLAxisAlignedBoundingBox.__typestr__,
        )
        self.assertArgIsBOOL(ModelIO.MDLCamera.frameBoundingBox_setNearAndFar_, 1)

        self.assertArgHasType(
            ModelIO.MDLCamera.lookAt_, 0, simd.vector_float3.__typestr__
        )

        self.assertArgHasType(
            ModelIO.MDLCamera.lookAt_from_, 0, simd.vector_float3.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLCamera.lookAt_from_, 1, simd.vector_float3.__typestr__
        )

        self.assertResultHasType(
            ModelIO.MDLCamera.rayTo_forViewPort_, simd.vector_float3.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLCamera.rayTo_forViewPort_, 0, simd.vector_int2.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLCamera.rayTo_forViewPort_, 1, simd.vector_int2.__typestr__
        )

        self.assertArgHasType(
            ModelIO.MDLCamera.bokehKernelWithSize_, 0, simd.vector_int2.__typestr__
        )

        self.assertResultHasType(
            ModelIO.MDLCamera.sensorEnlargement, simd.vector_float2.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLCamera.setSensorEnlargement_, 0, simd.vector_float2.__typestr__
        )

        self.assertResultHasType(
            ModelIO.MDLCamera.sensorShift, simd.vector_float2.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLCamera.setSensorShift_, 0, simd.vector_float2.__typestr__
        )

        self.assertResultHasType(
            ModelIO.MDLCamera.flash, simd.vector_float3.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLCamera.setFlash_, 0, simd.vector_float3.__typestr__
        )

        self.assertResultHasType(
            ModelIO.MDLCamera.exposureCompression, simd.vector_float2.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLCamera.setExposureCompression_, 0, simd.vector_float2.__typestr__
        )

        self.assertResultHasType(
            ModelIO.MDLCamera.exposure, simd.vector_float3.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLCamera.setExposure_, 0, simd.vector_float3.__typestr__
        )

        self.assertResultHasType(
            ModelIO.MDLStereoscopicCamera.leftViewMatrix,
            simd.matrix_float4x4.__typestr__,
        )
        self.assertResultHasType(
            ModelIO.MDLStereoscopicCamera.rightViewMatrix,
            simd.matrix_float4x4.__typestr__,
        )
        self.assertResultHasType(
            ModelIO.MDLStereoscopicCamera.leftProjectionMatrix,
            simd.matrix_float4x4.__typestr__,
        )
        self.assertResultHasType(
            ModelIO.MDLStereoscopicCamera.rightProjectionMatrix,
            simd.matrix_float4x4.__typestr__,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure
import ModelIO
from objc import simd


class TestMDLVoxelArray(TestCase):
    @expectedFailure
    def testStructs(self):
        self.assertEqual(
            ModelIO.MDLVoxelIndexExtent.__typestr__, b"{_MDLVoxelIndexExtent=<4i><4i>"
        )
        v = ModelIO.MDLVoxelIndexExtent()
        self.assertIs(v.minimumExtent, None)
        self.assertIs(v.maximumExtent, None)

    def testMethods(self):
        self.assertResultIsBOOL(
            ModelIO.MDLVoxelArray.voxelExistsAtIndex_allowAnyX_allowAnyY_allowAnyZ_allowAnyShell_
        )
        self.assertArgHasType(
            ModelIO.MDLVoxelArray.voxelExistsAtIndex_allowAnyX_allowAnyY_allowAnyZ_allowAnyShell_,
            0,
            simd.vector_int4.__typestr__,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLVoxelArray.voxelExistsAtIndex_allowAnyX_allowAnyY_allowAnyZ_allowAnyShell_,
            1,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLVoxelArray.voxelExistsAtIndex_allowAnyX_allowAnyY_allowAnyZ_allowAnyShell_,
            2,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLVoxelArray.voxelExistsAtIndex_allowAnyX_allowAnyY_allowAnyZ_allowAnyShell_,
            3,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLVoxelArray.voxelExistsAtIndex_allowAnyX_allowAnyY_allowAnyZ_allowAnyShell_,
            3,
        )

        self.assertArgHasType(
            ModelIO.MDLVoxelArray.setVoxelAtIndex_, 0, simd.vector_int4.__typestr__
        )

        self.assertArgHasType(
            ModelIO.MDLVoxelArray.initWithData_boundingBox_voxelExtent_,
            1,
            ModelIO.MDLAxisAlignedBoundingBox.__typestr__,
        )
        self.assertResultHasType(
            ModelIO.MDLVoxelArray.boundingBox,
            ModelIO.MDLAxisAlignedBoundingBox.__typestr__,
        )
        self.assertResultHasType(
            ModelIO.MDLVoxelArray.voxelBoundingBoxAtIndex_,
            ModelIO.MDLAxisAlignedBoundingBox.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLVoxelArray.voxelBoundingBoxAtIndex_,
            0,
            simd.vector_int4.__typestr__,
        )

        self.assertResultHasType(
            ModelIO.MDLVoxelArray.indexOfSpatialLocation_, simd.vector_int4.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLVoxelArray.indexOfSpatialLocation_,
            0,
            simd.vector_float3.__typestr__,
        )

        self.assertResultHasType(
            ModelIO.MDLVoxelArray.spatialLocationOfIndex_,
            simd.vector_float3.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLVoxelArray.spatialLocationOfIndex_,
            0,
            simd.vector_int4.__typestr__,
        )

        self.assertResultHasType(
            ModelIO.MDLVoxelArray.voxelIndexExtent,
            ModelIO.MDLVoxelIndexExtent.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLVoxelArray.voxelsWithinExtent_,
            0,
            ModelIO.MDLVoxelIndexExtent.__typestr__,
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(ModelIO.MDLVoxelArray.isValidSignedShellField)



from PyObjCTools.TestSupport import TestCase, min_os_level
import ModelIO
from objc import simd


class TestMDLCamera(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(ModelIO.MDLLightType)

    def testConstants(self):
        self.assertEqual(ModelIO.MDLLightTypeUnknown, 0)
        self.assertEqual(ModelIO.MDLLightTypeAmbient, 1)
        self.assertEqual(ModelIO.MDLLightTypeDirectional, 2)
        self.assertEqual(ModelIO.MDLLightTypeSpot, 3)
        self.assertEqual(ModelIO.MDLLightTypePoint, 4)
        self.assertEqual(ModelIO.MDLLightTypeLinear, 5)
        self.assertEqual(ModelIO.MDLLightTypeDiscArea, 6)
        self.assertEqual(ModelIO.MDLLightTypeRectangularArea, 7)
        self.assertEqual(ModelIO.MDLLightTypeSuperElliptical, 8)
        self.assertEqual(ModelIO.MDLLightTypePhotometric, 9)
        self.assertEqual(ModelIO.MDLLightTypeProbe, 10)
        self.assertEqual(ModelIO.MDLLightTypeEnvironment, 11)

    @min_os_level("10.11")
    def test_methods(self):
        self.assertArgHasType(
            ModelIO.MDLLight.irradianceAtPoint_, 0, simd.vector_float3.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLLight.irradianceAtPoint_colorSpace_,
            0,
            simd.vector_float3.__typestr__,
        )

        self.assertResultHasType(
            ModelIO.MDLAreaLight.superEllipticPower, simd.vector_float2.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLAreaLight.setSuperEllipticPower_,
            0,
            simd.vector_float2.__typestr__,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import ModelIO
from objc import simd


class TestMDLTexture(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(ModelIO.MDLTextureChannelEncoding)

    def testConstants(self):
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUInt8, 1)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUint8, 1)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUInt16, 2)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUint16, 2)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUInt24, 3)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUint24, 3)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUInt32, 4)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUint32, 4)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingFloat16, 0x102)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingFloat16SR, 0x302)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingFloat32, 0x104)

    def testMethods(self):
        self.assertArgIsBOOL(
            ModelIO.MDLTexture.initWithData_topLeftOrigin_name_dimensions_rowStride_channelCount_channelEncoding_isCube_,
            1,
        )
        self.assertArgHasType(
            ModelIO.MDLTexture.initWithData_topLeftOrigin_name_dimensions_rowStride_channelCount_channelEncoding_isCube_,
            3,
            simd.vector_int2.__typestr__,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLTexture.initWithData_topLeftOrigin_name_dimensions_rowStride_channelCount_channelEncoding_isCube_,
            7,
        )

        self.assertResultIsBOOL(ModelIO.MDLTexture.writeToURL_)
        self.assertResultIsBOOL(ModelIO.MDLTexture.writeToURL_type_)

        self.assertArgIsBOOL(
            ModelIO.MDLTexture.texelDataWithTopLeftOriginAtMipLevel_create_, 1
        )
        self.assertArgIsBOOL(
            ModelIO.MDLTexture.texelDataWithBottomLeftOriginAtMipLevel_create_, 1
        )

        self.assertResultIsBOOL(ModelIO.MDLTexture.isCube)
        self.assertArgIsBOOL(ModelIO.MDLTexture.setIsCube_, 0)

        self.assertArgHasType(
            ModelIO.MDLNoiseTexture.initVectorNoiseWithSmoothness_name_textureDimensions_channelEncoding_,
            2,
            simd.vector_int2.__typestr__,
        )

        self.assertArgHasType(
            ModelIO.MDLNoiseTexture.initScalarNoiseWithSmoothness_name_textureDimensions_channelCount_channelEncoding_grayscale_,
            2,
            simd.vector_int2.__typestr__,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLNoiseTexture.initScalarNoiseWithSmoothness_name_textureDimensions_channelCount_channelEncoding_grayscale_,
            5,
        )

        self.assertArgHasType(
            ModelIO.MDLNoiseTexture.initCellularNoiseWithFrequency_name_textureDimensions_channelEncoding_,
            2,
            simd.vector_int2.__typestr__,
        )

        self.assertResultHasType(
            ModelIO.MDLTexture.dimensions, simd.vector_int2.__typestr__
        )

        self.assertArgHasType(
            ModelIO.MDLTexture.irradianceTextureCubeWithTexture_name_dimensions_,
            2,
            simd.vector_int2.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLTexture.irradianceTextureCubeWithTexture_name_dimensions_roughness_,
            2,
            simd.vector_int2.__typestr__,
        )

        self.assertArgHasType(
            ModelIO.MDLCheckerboardTexture.initWithDivisions_name_dimensions_channelCount_channelEncoding_color1_color2_,
            2,
            simd.vector_int2.__typestr__,
        )

        self.assertArgHasType(
            ModelIO.MDLSkyCubeTexture.initWithName_channelEncoding_textureDimensions_turbidity_sunElevation_upperAtmosphereScattering_groundAlbedo_,
            2,
            simd.vector_int2.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLSkyCubeTexture.initWithName_channelEncoding_textureDimensions_turbidity_sunElevation_sunAzimuth_upperAtmosphereScattering_groundAlbedo_,
            2,
            simd.vector_int2.__typestr__,
        )

        self.assertResultHasType(
            ModelIO.MDLSkyCubeTexture.highDynamicRangeCompression,
            simd.vector_float2.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLSkyCubeTexture.setHighDynamicRangeCompression_,
            0,
            simd.vector_float2.__typestr__,
        )

        self.assertArgHasType(
            ModelIO.MDLColorSwatchTexture.initWithColorTemperatureGradientFrom_toColorTemperature_name_textureDimensions_,
            3,
            simd.vector_int2.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLColorSwatchTexture.initWithColorGradientFrom_toColor_name_textureDimensions_,
            3,
            simd.vector_int2.__typestr__,
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(ModelIO.MDLTexture.hasAlphaValues)
        self.assertArgIsBOOL(ModelIO.MDLTexture.setHasAlphaValues_, 0)

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(ModelIO.MDLTexture.writeToURL_level_)
        self.assertResultIsBOOL(ModelIO.MDLTexture.writeToURL_type_level_)



from PyObjCTools.TestSupport import TestCase, min_os_level
import ModelIO
from objc import simd


class TestMDLMesh(TestCase):
    def testMethods(self):
        self.assertArgHasType(
            ModelIO.MDLMesh.newBoxWithDimensions_segments_geometryType_inwardNormals_allocator_,
            0,
            simd.vector_float3.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLMesh.newBoxWithDimensions_segments_geometryType_inwardNormals_allocator_,
            1,
            simd.vector_uint3.__typestr__,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLMesh.newBoxWithDimensions_segments_geometryType_inwardNormals_allocator_,
            3,
        )

        self.assertArgHasType(
            ModelIO.MDLMesh.newEllipsoidWithRadii_radialSegments_verticalSegments_geometryType_inwardNormals_hemisphere_allocator_,
            0,
            simd.vector_float3.__typestr__,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLMesh.newEllipsoidWithRadii_radialSegments_verticalSegments_geometryType_inwardNormals_hemisphere_allocator_,
            4,
        )  # noqa: B950

        self.assertArgHasType(
            ModelIO.MDLMesh.newCylinderWithHeight_radii_radialSegments_verticalSegments_geometryType_inwardNormals_allocator_,
            1,
            simd.vector_float2.__typestr__,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLMesh.newCylinderWithHeight_radii_radialSegments_verticalSegments_geometryType_inwardNormals_allocator_,
            5,
        )  # noqa: B950

        self.assertArgHasType(
            ModelIO.MDLMesh.newEllipticalConeWithHeight_radii_radialSegments_verticalSegments_geometryType_inwardNormals_allocator_,
            1,
            simd.vector_float2.__typestr__,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLMesh.newEllipticalConeWithHeight_radii_radialSegments_verticalSegments_geometryType_inwardNormals_allocator_,
            5,
        )  # noqa: B950

        self.assertArgHasType(
            ModelIO.MDLMesh.newPlaneWithDimensions_segments_geometryType_allocator_,
            0,
            simd.vector_float2.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLMesh.newPlaneWithDimensions_segments_geometryType_allocator_,
            1,
            simd.vector_uint2.__typestr__,
        )

        self.assertArgIsBOOL(
            ModelIO.MDLMesh.newIcosahedronWithRadius_inwardNormals_allocator_, 1
        )

        self.assertResultIsBOOL(
            ModelIO.MDLMesh.generateAmbientOcclusionTextureWithSize_raysPerSample_attenuationFactor_objectsToConsider_vertexAttributeNamed_materialPropertyNamed_
        )  # noqa: B950
        self.assertArgHasType(
            ModelIO.MDLMesh.generateAmbientOcclusionTextureWithSize_raysPerSample_attenuationFactor_objectsToConsider_vertexAttributeNamed_materialPropertyNamed_,
            0,
            simd.vector_int2.__typestr__,
        )  # noqa: B950

        self.assertResultIsBOOL(
            ModelIO.MDLMesh.generateAmbientOcclusionVertexColorsWithRaysPerSample_attenuationFactor_objectsToConsider_vertexAttributeNamed_
        )  # noqa: B950

        self.assertResultIsBOOL(
            ModelIO.MDLMesh.generateAmbientOcclusionVertexColorsWithRaysPerSample_attenuationFactor_objectsToConsider_vertexAttributeNamed_
        )  # noqa: B950

        self.assertResultIsBOOL(
            ModelIO.MDLMesh.generateLightMapTextureWithTextureSize_lightsToConsider_objectsToConsider_vertexAttributeNamed_materialPropertyNamed_
        )  # noqa: B950
        self.assertArgHasType(
            ModelIO.MDLMesh.generateLightMapTextureWithTextureSize_lightsToConsider_objectsToConsider_vertexAttributeNamed_materialPropertyNamed_,
            0,
            simd.vector_int2.__typestr__,
        )  # noqa: B950

        self.assertResultIsBOOL(
            ModelIO.MDLMesh.generateLightMapTextureWithQuality_lightsToConsider_objectsToConsider_vertexAttributeNamed_materialPropertyNamed_
        )  # noqa: B950

        self.assertResultIsBOOL(
            ModelIO.MDLMesh.generateLightMapVertexColorsWithLightsToConsider_objectsToConsider_vertexAttributeNamed_
        )

        self.assertResultHasType(
            ModelIO.MDLMesh.boundingBox, ModelIO.MDLAxisAlignedBoundingBox.__typestr__
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertArgHasType(
            ModelIO.MDLMesh.initBoxWithExtent_segments_inwardNormals_geometryType_allocator_,
            0,
            simd.vector_float3.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLMesh.initBoxWithExtent_segments_inwardNormals_geometryType_allocator_,
            1,
            simd.vector_uint3.__typestr__,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLMesh.initBoxWithExtent_segments_inwardNormals_geometryType_allocator_,
            2,
        )

        self.assertArgHasType(
            ModelIO.MDLMesh.initSphereWithExtent_segments_inwardNormals_geometryType_allocator_,
            0,
            simd.vector_float3.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLMesh.initSphereWithExtent_segments_inwardNormals_geometryType_allocator_,
            1,
            simd.vector_uint2.__typestr__,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLMesh.initSphereWithExtent_segments_inwardNormals_geometryType_allocator_,
            2,
        )

        self.assertArgIsBOOL(
            ModelIO.MDLMesh.initHemisphereWithExtent_segments_inwardNormals_cap_geometryType_allocator_,
            2,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLMesh.initHemisphereWithExtent_segments_inwardNormals_cap_geometryType_allocator_,
            3,
        )

        self.assertArgHasType(
            ModelIO.MDLMesh.initCylinderWithExtent_segments_inwardNormals_topCap_bottomCap_geometryType_allocator_,
            0,
            simd.vector_float3.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLMesh.initCylinderWithExtent_segments_inwardNormals_topCap_bottomCap_geometryType_allocator_,
            1,
            simd.vector_uint2.__typestr__,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLMesh.initCylinderWithExtent_segments_inwardNormals_topCap_bottomCap_geometryType_allocator_,
            2,
        )  # noqa: B950
        self.assertArgIsBOOL(
            ModelIO.MDLMesh.initCylinderWithExtent_segments_inwardNormals_topCap_bottomCap_geometryType_allocator_,
            3,
        )  # noqa: B950
        self.assertArgIsBOOL(
            ModelIO.MDLMesh.initCylinderWithExtent_segments_inwardNormals_topCap_bottomCap_geometryType_allocator_,
            4,
        )  # noqa: B950

        self.assertArgHasType(
            ModelIO.MDLMesh.initCapsuleWithExtent_cylinderSegments_hemisphereSegments_inwardNormals_geometryType_allocator_,
            0,
            simd.vector_float3.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLMesh.initCapsuleWithExtent_cylinderSegments_hemisphereSegments_inwardNormals_geometryType_allocator_,
            1,
            simd.vector_uint2.__typestr__,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLMesh.initCapsuleWithExtent_cylinderSegments_hemisphereSegments_inwardNormals_geometryType_allocator_,
            3,
        )  # noqa: B950

        self.assertArgHasType(
            ModelIO.MDLMesh.initConeWithExtent_segments_inwardNormals_cap_geometryType_allocator_,
            0,
            simd.vector_float3.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLMesh.initConeWithExtent_segments_inwardNormals_cap_geometryType_allocator_,
            1,
            simd.vector_uint2.__typestr__,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLMesh.initConeWithExtent_segments_inwardNormals_cap_geometryType_allocator_,
            3,
        )

        self.assertArgHasType(
            ModelIO.MDLMesh.initPlaneWithExtent_segments_geometryType_allocator_,
            0,
            simd.vector_float3.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLMesh.initPlaneWithExtent_segments_geometryType_allocator_,
            1,
            simd.vector_uint2.__typestr__,
        )

        self.assertArgHasType(
            ModelIO.MDLMesh.initIcosahedronWithExtent_inwardNormals_geometryType_allocator_,
            0,
            simd.vector_float3.__typestr__,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLMesh.initIcosahedronWithExtent_inwardNormals_geometryType_allocator_,
            1,
        )

        self.assertArgHasType(
            ModelIO.MDLMesh.newCapsuleWithHeight_radii_radialSegments_verticalSegments_hemisphereSegments_geometryType_inwardNormals_allocator_,
            1,
            simd.vector_float2.__typestr__,
        )  # noqa: B950
        self.assertArgIsBOOL(
            ModelIO.MDLMesh.newCapsuleWithHeight_radii_radialSegments_verticalSegments_hemisphereSegments_geometryType_inwardNormals_allocator_,
            6,
        )  # noqa: B950

        pass

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsOut(ModelIO.MDLMesh.makeVerticesUniqueAndReturnError_, 0)



from PyObjCTools.TestSupport import TestCase, min_sdk_level, min_os_level
import ModelIO
from objc import simd


class TestMDLTransformStackHelper(ModelIO.NSObject):
    def float4x4AtTime_(self, a):
        return 1

    def double4x4AtTime_(self, a):
        return 1


class TestMDLTransformStack(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(ModelIO.MDLTransformOpRotationOrder)

    def testConstants(self):
        self.assertEqual(ModelIO.MDLTransformOpRotationOrderXYZ, 1)
        self.assertEqual(ModelIO.MDLTransformOpRotationOrderXZY, 2)
        self.assertEqual(ModelIO.MDLTransformOpRotationOrderYXZ, 3)
        self.assertEqual(ModelIO.MDLTransformOpRotationOrderYZX, 4)
        self.assertEqual(ModelIO.MDLTransformOpRotationOrderZXY, 5)
        self.assertEqual(ModelIO.MDLTransformOpRotationOrderZYX, 6)

    @min_sdk_level("10.13")
    def testProtocols(self):
        self.assertProtocolExists("MDLTransformOp")

    def test_protocol_methods(self):
        self.assertResultHasType(
            TestMDLTransformStackHelper.float4x4AtTime_,
            simd.matrix_float4x4.__typestr__,
        )
        self.assertResultHasType(
            TestMDLTransformStackHelper.double4x4AtTime_,
            simd.matrix_double4x4.__typestr__,
        )

    @min_os_level("10.13")
    def test_methods10_13(self):
        self.assertResultHasType(
            ModelIO.MDLTransformStack.float4x4AtTime_, simd.matrix_float4x4.__typestr__
        )
        self.assertResultHasType(
            ModelIO.MDLTransformStack.double4x4AtTime_,
            simd.matrix_double4x4.__typestr__,
        )



from PyObjCTools.TestSupport import TestCase, min_sdk_level
import ModelIO
from objc import simd


class TestMDLAnimation(TestCase):
    @min_sdk_level("10.13")
    def testProtocols(self):
        self.assertProtocolExists("MDLJointAnimation")

    @min_sdk_level("10.13")
    def test_methods(self):
        self.assertResultHasType(
            ModelIO.MDLAnimationBindComponent.geometryBindTransform,
            simd.matrix_double4x4.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLAnimationBindComponent.setGeometryBindTransform_,
            0,
            simd.matrix_double4x4.__typestr__,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc
import ModelIO
from objc import simd


class TestMDLTransformHelper(ModelIO.NSObject):
    def matrix(self):
        return 1

    def setMatrix_(self, v):
        return 1

    def resetsTransform(self):
        return 1

    def setResetsTransform_(self, v):
        return 1

    def minimumTime(self):
        return 1

    def setMinimumTime_(self, v):
        return 1

    def maximumTime(self):
        return 1

    def setMaximumTime_(self, v):
        return 1

    def setLocalTransform_forTime_(self, t, tm):
        pass

    def setLocalTransform_(self, t):
        pass

    def localTransformAtTime_(self, t):
        return 1

    def globalTransformWithObject_atTime_(self, o, t):
        return 1


class TestMDLTransform(TestCase):
    def testMethods(self):
        self.assertResultHasType(
            ModelIO.TestMDLTransformHelper.matrix, simd.matrix_float4x4.__typestr__
        )
        self.assertArgHasType(
            ModelIO.TestMDLTransformHelper.setMatrix_,
            0,
            simd.matrix_float4x4.__typestr__,
        )

        self.assertResultIsBOOL(ModelIO.TestMDLTransformHelper.resetsTransform)
        self.assertArgIsBOOL(ModelIO.TestMDLTransformHelper.setResetsTransform_, 0)

        self.assertResultHasType(
            ModelIO.TestMDLTransformHelper.minimumTime, objc._C_DBL
        )
        self.assertArgHasType(
            ModelIO.TestMDLTransformHelper.setMinimumTime_, 0, objc._C_DBL
        )

        self.assertResultHasType(
            ModelIO.TestMDLTransformHelper.maximumTime, objc._C_DBL
        )
        self.assertArgHasType(
            ModelIO.TestMDLTransformHelper.setMaximumTime_, 0, objc._C_DBL
        )

        self.assertArgHasType(
            ModelIO.TestMDLTransformHelper.setLocalTransform_forTime_,
            0,
            simd.matrix_float4x4.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.TestMDLTransformHelper.setLocalTransform_forTime_, 1, objc._C_DBL
        )

        self.assertArgHasType(
            ModelIO.TestMDLTransformHelper.setLocalTransform_,
            0,
            simd.matrix_float4x4.__typestr__,
        )

        self.assertResultHasType(
            ModelIO.TestMDLTransformHelper.localTransformAtTime_,
            simd.matrix_float4x4.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.TestMDLTransformHelper.localTransformAtTime_, 0, objc._C_DBL
        )

        self.assertResultHasType(
            ModelIO.TestMDLTransformHelper.globalTransformWithObject_atTime_,
            simd.matrix_float4x4.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.TestMDLTransformHelper.globalTransformWithObject_atTime_,
            1,
            objc._C_DBL,
        )

        self.assertArgHasType(
            ModelIO.MDLTransform.initWithMatrix_, 0, simd.matrix_float4x4.__typestr__
        )

        self.assertArgHasType(
            ModelIO.MDLTransform.initWithMatrix_resetsTransform_,
            0,
            simd.matrix_float4x4.__typestr__,
        )
        self.assertArgIsBOOL(ModelIO.MDLTransform.initWithMatrix_resetsTransform_, 1)

        self.assertResultHasType(
            ModelIO.MDLTransform.translationAtTime_, simd.vector_float3.__typestr__
        )
        self.assertResultHasType(
            ModelIO.MDLTransform.rotationAtTime_, simd.vector_float3.__typestr__
        )
        self.assertResultHasType(
            ModelIO.MDLTransform.shearAtTime_, simd.vector_float3.__typestr__
        )
        self.assertResultHasType(
            ModelIO.MDLTransform.scaleAtTime_, simd.vector_float3.__typestr__
        )

        self.assertArgHasType(
            ModelIO.MDLTransform.setMatrix_forTime_, 0, simd.matrix_float4x4.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLTransform.setTranslation_forTime_,
            0,
            simd.vector_float3.__typestr__,
        )
        self.assertArgHasType(
            ModelIO.MDLTransform.setRotation_forTime_, 0, simd.vector_float3.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLTransform.setShear_forTime_, 0, simd.vector_float3.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLTransform.setScale_forTime_, 0, simd.vector_float3.__typestr__
        )

        self.assertResultHasType(
            ModelIO.MDLTransform.rotationMatrixAtTime_, simd.matrix_float4x4.__typestr__
        )

        self.assertResultHasType(
            ModelIO.MDLTransform.translation, simd.vector_float3.__typestr__
        )
        self.assertResultHasType(
            ModelIO.MDLTransform.rotation, simd.vector_float3.__typestr__
        )
        self.assertResultHasType(
            ModelIO.MDLTransform.shear, simd.vector_float3.__typestr__
        )
        self.assertResultHasType(
            ModelIO.MDLTransform.scale, simd.vector_float3.__typestr__
        )

        self.assertArgHasType(
            ModelIO.MDLTransform.setTranslation_, 0, simd.vector_float3.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLTransform.setRotation_, 0, simd.vector_float3.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLTransform.setShear_, 0, simd.vector_float3.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLTransform.setScale_, 0, simd.vector_float3.__typestr__
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertArgIsBOOL(
            ModelIO.MDLTransform.initWithTransformComponent_resetsTransform_, 1
        )

    def testProtocolObjects(self):
        self.assertProtocolExists("MDLTransformComponent")



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc
import ModelIO
from objc import simd


class TestMDLAssetHelper(ModelIO.NSObject):
    def sphericalHarmonicsLevel(self):
        return 1

    def setSphericalHarmonicsLevel_(self, v):
        pass

    def boundingBox(self):
        return 1

    def setBoundingBox_(self, v):
        pass

    def sphericalHarmonicsCoefficientsAtPosition_(self, v):
        return 1


class TestMDLAsset(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(
            ModelIO.MDLAsset.initWithURL_vertexDescriptor_bufferAllocator_preserveTopology_error_,
            3,
        )
        self.assertArgIsOut(
            ModelIO.MDLAsset.initWithURL_vertexDescriptor_bufferAllocator_preserveTopology_error_,
            4,
        )

        self.assertResultIsBOOL(ModelIO.MDLAsset.exportAssetToURL_)

        self.assertResultIsBOOL(ModelIO.MDLAsset.exportAssetToURL_error_)
        self.assertArgIsOut(ModelIO.MDLAsset.exportAssetToURL_error_, 1)

        self.assertResultIsBOOL(ModelIO.MDLAsset.canImportFileExtension_)

        self.assertResultIsBOOL(ModelIO.MDLAsset.canExportFileExtension_)

        self.assertResultHasType(
            TestMDLAssetHelper.boundingBox,
            ModelIO.MDLAxisAlignedBoundingBox.__typestr__,
        )
        self.assertArgHasType(
            TestMDLAssetHelper.setBoundingBox_,
            0,
            ModelIO.MDLAxisAlignedBoundingBox.__typestr__,
        )
        self.assertArgHasType(
            TestMDLAssetHelper.sphericalHarmonicsCoefficientsAtPosition_,
            0,
            simd.vector_float3.__typestr__,
        )

        self.assertResultHasType(
            TestMDLAssetHelper.sphericalHarmonicsLevel, objc._C_NSUInteger
        )
        self.assertArgHasType(
            TestMDLAssetHelper.setSphericalHarmonicsLevel_, 0, objc._C_NSUInteger
        )

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsBOOL(
            ModelIO.MDLAsset.initWithURL_bufferAllocator_preserveIndexing_error_, 2
        )
        self.assertArgIsOut(
            ModelIO.MDLAsset.initWithURL_bufferAllocator_preserveIndexing_error_, 3
        )

        self.assertResultHasType(
            ModelIO.MDLAsset.upAxis, simd.vector_float3.__typestr__
        )
        self.assertArgHasType(
            ModelIO.MDLAsset.setUpAxis_, 0, simd.vector_float3.__typestr__
        )

    def testProtocolObjects(self):
        self.assertProtocolExists("MDLLightProbeIrradianceDataSource")
